Package: AIPW
Title: Augmented Inverse Probability Weighting
Version: 0.6.9.3
Authors@R: 
    c(person(given = "Yongqi",
             family = "Zhong",
             role = c("aut", "cre"),
             email = "yq.zhong7@gmail.com",
             comment = c(ORCID = "0000-0002-4042-7450")),
      person(given = "Ashley",
             family = "Naimi",
             role = c("aut"),
             email = "ashley.naimi@emory.edu",
             comment = c(ORCID = "0000-0002-1510-8175")),
      person(given = "Gabriel",
             family = "Conzuelo",
             role = c("ctb"),
             email = "gabriel.conzuelo@pitt.edu"),
      person(given = "Edward",
             family = "Kennedy",
             role = c("ctb"),
             email = "edward@stat.cmu.edu"))
Maintainer: Yongqi Zhong <yq.zhong7@gmail.com>
Description: The 'AIPW' package implements the augmented inverse probability weighting, a doubly robust estimator, for average causal effect estimation with user-defined stacked machine learning algorithms. To cite the 'AIPW' package, please use: "Yongqi Zhong, Edward H. Kennedy, Lisa M. Bodnar, Ashley I. Naimi (2021). AIPW: An R Package for Augmented Inverse Probability Weighted Estimation of Average Causal Effects. American Journal of Epidemiology. <doi:10.1093/aje/kwab207>". Visit: <https://yqzhong7.github.io/AIPW/> for more information.
License: GPL-3
Encoding: UTF-8
Language: en
LazyData: true
Suggests: testthat (>= 2.1.0), knitr, rmarkdown, covr, tmle
RoxygenNote: 7.2.2
Imports: stats, utils, R6, SuperLearner, ggplot2, future.apply,
        progressr, Rsolnp
URL: https://github.com/yqzhong7/AIPW
BugReports: https://github.com/yqzhong7/AIPW/issues
VignetteBuilder: knitr
Depends: R (>= 2.10)
NeedsCompilation: no
Packaged: 2026-04-11 17:07:51 UTC; k
Author: Yongqi Zhong [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-4042-7450>),
  Ashley Naimi [aut] (ORCID: <https://orcid.org/0000-0002-1510-8175>),
  Gabriel Conzuelo [ctb],
  Edward Kennedy [ctb]
Repository: CRAN
Date/Publication: 2026-04-12 05:10:34 UTC
Built: R 4.5.2; ; 2026-04-12 06:31:33 UTC; unix
